package org.fortiss.tooling.kernel.ui.service;

import static org.fortiss.tooling.kernel.ui.ToolingKernelUIActivator.getService;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.model.constraints.IConstraint;
import org.fortiss.tooling.kernel.model.constraints.IConstraintVerificationStatus;
import org.fortiss.tooling.kernel.service.IConstraintVerificationService;
import org.fortiss.tooling.kernel.utils.ServicesUtils;

/**
 * GUI aspects of {@link IConstraintVerificationService}.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: FE097BC16B9814644F07C46D58749858
 */
public interface IConstraintVerificationUIService {
	/**
	 * Returns the service instance.
	 * 
	 * @deprecated use {@link ServicesUtils#getService(org.osgi.framework.Bundle, Class)} and
	 *             {@link ServicesUtils#ungetService(org.osgi.framework.Bundle, Class)}, instead.
	 *             See also {@link ToolingKernelActivator#getService(Class)} and
	 *             {@link ToolingKernelActivator#ungetService(Class)}
	 */
	@Deprecated
	public static IConstraintVerificationUIService getInstance() {
		return getService(IConstraintVerificationUIService.class);
	}

	/** Action to take when trying to open the given status. */
	public void openStatus(IConstraintVerificationStatus status);

	/** True if the given status can be open. */
	public boolean canOpen(IConstraintVerificationStatus status);

	/** Install the given constraint: install notifiers and triggers an "onOutdate" event. */
	public void install(IConstraint constraint);

	/**
	 * The following features deal with the usage of constraints to drive the process.
	 * 
	 * EXPERIMENTAL.
	 */

	/** Activates the constraint type. */
	public <T extends IConstraint> void activate(String id, EObject context);

	/** Deactivates the constraint type. */
	public <T extends IConstraint> void deactivate(String id, EObject context);
}
