/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import static org.fortiss.tooling.kernel.ToolingKernel.printPhase;
import static org.fortiss.tooling.kernel.ToolingKernelActivator.getService;
import static org.fortiss.tooling.kernel.ToolingKernelActivator.ungetService;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.IConnectionCompositor;
import org.fortiss.tooling.kernel.extension.IConstraintChecker;
import org.fortiss.tooling.kernel.extension.IConstraintVerifier;
import org.fortiss.tooling.kernel.extension.IElementCompositor;
import org.fortiss.tooling.kernel.extension.ILibraryElementHandler;
import org.fortiss.tooling.kernel.extension.IMigrationProvider;
import org.fortiss.tooling.kernel.extension.IPrototypeProvider;
import org.fortiss.tooling.kernel.extension.IStorageProvider;
import org.fortiss.tooling.kernel.extension.ITransformationProvider;
import org.fortiss.tooling.kernel.extension.LogMessageHandler;
import org.fortiss.tooling.kernel.extension.data.LogMessage;
import org.fortiss.tooling.kernel.internal.storage.eclipse.EclipseResourceStorageService;
import org.fortiss.tooling.kernel.model.constraints.IConstraint;
import org.fortiss.tooling.kernel.service.IConnectionCompositorService;
import org.fortiss.tooling.kernel.service.IConstraintCheckerService;
import org.fortiss.tooling.kernel.service.IConstraintVerificationService;
import org.fortiss.tooling.kernel.service.IEclipseResourceStorageService;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.service.ILibraryService;
import org.fortiss.tooling.kernel.service.ILoggingService;
import org.fortiss.tooling.kernel.service.IMigrationService;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.service.IPrototypeService;
import org.fortiss.tooling.kernel.service.ITransformationService;
import org.fortiss.tooling.kernel.service.ITutorialService;

/**
 * Implementation of the kernel initialization and startup.
 * 
 * @author hoelzl
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating YELLOW Hash: 11DFD3BDA7E63E3C5F9ACA49BA921A4F
 */
public final class ToolingKernelInternal {
	/** Initialize the kernel services. */
	// FIXME (FH): https://af3-developer.fortiss.org/issues/2590
	// initialization no longer needed after extension points are gone
	public static void initializeKernel() {
		printPhase("Tooling Kernel initializing ...");
		IConnectionCompositorService ccs = getService(IConnectionCompositorService.class);
		((ConnectionCompositorService)ccs).initializeService();
		ungetService(IConnectionCompositorService.class);

		IConstraintCheckerService cchs = getService(IConstraintCheckerService.class);
		((ConstraintCheckerService)cchs).initializeService();
		ungetService(IConstraintCheckerService.class);

		IConstraintVerificationService cvs = getService(IConstraintVerificationService.class);
		((ConstraintVerificationService)cvs).initializeService();
		ungetService(IConstraintVerificationService.class);

		IElementCompositorService ecs = getService(IElementCompositorService.class);
		((ElementCompositorService)ecs).initializeService();
		ungetService(IElementCompositorService.class);

		IEclipseResourceStorageService erss = getService(IEclipseResourceStorageService.class);
		((EclipseResourceStorageService)erss).initializeService();
		ungetService(IEclipseResourceStorageService.class);

		ILibraryService libs = getService(ILibraryService.class);
		((LibraryService)libs).initializeService();
		ungetService(ILibraryService.class);

		ILoggingService logs = getService(ILoggingService.class);
		((LoggingService)logs).initializeService();
		ungetService(ILoggingService.class);

		IMigrationService migs = getService(IMigrationService.class);
		((MigrationService)migs).initializeService();
		ungetService(IMigrationService.class);

		IPersistencyService ps = getService(IPersistencyService.class);
		((PersistencyService)ps).initializeService();
		ungetService(IPersistencyService.class);

		IPrototypeService protos = getService(IPrototypeService.class);
		((PrototypeService)protos).initializeService();
		ungetService(IPrototypeService.class);

		ITransformationService trans = getService(ITransformationService.class);
		((TransformationService)trans).initializeService();
		ungetService(ITransformationService.class);

		ITutorialService tuts = getService(ITutorialService.class);
		((TutorialService)tuts).initializeService();
		ungetService(ITutorialService.class);
		printPhase("Tooling Kernel initialized.");
	}

	/** Starts the kernel services. */
	// FIXME (FH): https://af3-developer.fortiss.org/issues/2590
	// starting no longer needed after extension points are gone
	public static void startKernel() {
		printPhase("Tooling Kernel starting ...");
		IConnectionCompositorService ccs = getService(IConnectionCompositorService.class);
		((ConnectionCompositorService)ccs).startService();
		ungetService(IConnectionCompositorService.class);

		IConstraintCheckerService cchs = getService(IConstraintCheckerService.class);
		((ConstraintCheckerService)cchs).startService();
		ungetService(IConstraintCheckerService.class);

		IConstraintVerificationService cvs = getService(IConstraintVerificationService.class);
		((ConstraintVerificationService)cvs).startService();
		ungetService(IConstraintVerificationService.class);

		IElementCompositorService ecs = getService(IElementCompositorService.class);
		((ElementCompositorService)ecs).startService();
		ungetService(IElementCompositorService.class);

		IEclipseResourceStorageService erss = getService(IEclipseResourceStorageService.class);
		((EclipseResourceStorageService)erss).startService();
		ungetService(IEclipseResourceStorageService.class);

		ILibraryService libs = getService(ILibraryService.class);
		((LibraryService)libs).startService();
		ungetService(ILibraryService.class);

		ILoggingService logs = getService(ILoggingService.class);
		((LoggingService)logs).startService();
		ungetService(ILoggingService.class);

		IMigrationService migs = getService(IMigrationService.class);
		((MigrationService)migs).startService();
		ungetService(IMigrationService.class);

		IPersistencyService ps = getService(IPersistencyService.class);
		((PersistencyService)ps).startService();
		ungetService(IPersistencyService.class);

		IPrototypeService protos = getService(IPrototypeService.class);
		((PrototypeService)protos).startService();
		ungetService(IPrototypeService.class);

		ITransformationService trans = getService(ITransformationService.class);
		((TransformationService)trans).startService();
		ungetService(ITransformationService.class);

		ITutorialService tuts = getService(ITutorialService.class);
		((TutorialService)tuts).startService();
		ungetService(ITutorialService.class);
		printPhase("Tooling Kernel started.");
	}

	/** Registers the given transformation with the kernel. */
	public static void registerTransformationProvider(ITransformationProvider provider,
			Class<?> sourceClass, Class<?> targetClass) {
		ITransformationService trans = getService(ITransformationService.class);
		((TransformationService)trans).registerTransformationProvider(provider, sourceClass,
				targetClass);
		ungetService(ITransformationService.class);
	}

	/** Registers the given provider with the kernel. */
	public static void registerPrototypeProvider(IPrototypeProvider provider) {
		IPrototypeService protos = getService(IPrototypeService.class);
		((PrototypeService)protos).registerPrototypeProvider(provider);
		ungetService(IPrototypeService.class);
	}

	/** Registers the given storage provider with the kernel. */
	public static void registerModelStorageProvider(IStorageProvider provider) {
		IPersistencyService ps = getService(IPersistencyService.class);
		((PersistencyService)ps).registerModelStorageProvider(provider);
		ungetService(IPersistencyService.class);
	}

	/** Registers the migration provider with the service. */
	public static void registerMigrationProvider(IMigrationProvider provider,
			Class<?> modelElementClass) {
		IMigrationService migs = getService(IMigrationService.class);
		((MigrationService)migs).registerMigrationProvider(provider, modelElementClass);
		ungetService(IMigrationService.class);
	}

	/** Registers the given handler with the kernel. */
	public static void registerLibraryElementHandler(ILibraryElementHandler<EObject> handler,
			Class<?> modelElementClass) {
		ILibraryService ls = getService(ILibraryService.class);
		ls.registerLibraryElementHandler(handler, modelElementClass);
		ungetService(ILibraryService.class);
	}

	/** Registers the given handler with the kernel. */
	public static <LM extends LogMessage> void registerLogMessageHandler(
			LogMessageHandler<LM> handler, Class<LM> logMessageClass) {
		ILoggingService ls = getService(ILoggingService.class);
		ls.registerLogMessageHandler(handler, logMessageClass);
		ungetService(ILoggingService.class);
	}

	/** Registers the given compositor with the kernel. */
	public static void registerElementCompositor(IElementCompositor<EObject> compositor,
			Class<?> modelElementClass) {
		IElementCompositorService ecs = getService(IElementCompositorService.class);
		ecs.registerElementCompositor(compositor, modelElementClass);
		ungetService(IElementCompositorService.class);
	}

	/** Registers the given verifier with the kernel. */
	public static <T extends IConstraint> void registerConstraintVerifier(
			IConstraintVerifier<T> verifier, Class<T> constraintElementClass) {
		IConstraintVerificationService cvs = getService(IConstraintVerificationService.class);
		cvs.registerConstraintVerifier(verifier, constraintElementClass);
		ungetService(IConstraintVerificationService.class);
	}

	/** Registers the given constraint checker with the kernel. */
	public static void registerConstraintChecker(IConstraintChecker<EObject> checker,
			Class<?> modelElementClass) {
		IConstraintCheckerService ccs = getService(IConstraintCheckerService.class);
		ccs.registerConstraintChecker(checker, modelElementClass);
		ungetService(IConstraintCheckerService.class);
	}

	/** Registers the given compositor with the kernel. */
	public static void registerConnectionCompositor(
			IConnectionCompositor<EObject, EObject, EObject> compositor, Class<?> sourceClass,
			Class<?> targetClass) {
		IConnectionCompositorService ccs = getService(IConnectionCompositorService.class);
		ccs.registerConnectionCompositor(compositor, sourceClass, targetClass);
		ungetService(IConnectionCompositorService.class);
	}
}
